# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2017-12-05 07:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('payfast', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='payfastorder',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='payfastorder',
            name='updated_at',
            field=models.DateTimeField(auto_now=True),
        ),
    ]
